@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_auth.c,v 1.15 2004/08/27 12:25:27 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_auth.c,v 1.6 2005/10/22 12:57:57 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003, 2004	Tatsuo Ishii
+ * Copyright (c) 2003-2005	Tatsuo Ishii
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -109,7 +109,9 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 		return -1;
 	}
 
-	if (REPLICATION)
+	pid = ntohl(pid);
+
+	if (DUAL_MODE)
 	{
 		status = pool_read(SECONDARY(cp), &pid1, sizeof(pid1));
 
@@ -118,9 +120,15 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 			pool_error("pool_do_auth: read authentication kind from secondary failed");
 			return -1;
 		}
-	}
 
-	pid = ntohl(pid);
+		pid1 = ntohl(pid1);
+
+		if (pid != pid1)
+		{
+			pool_error("pool_do_auth: authentication kind does not match between master(%d) and secondary(%d)", pid, pid1);
+			return -1;
+		}
+	}
 
 	/* trust? */
 	if (pid == 0)
@@ -148,7 +156,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 
 		pid = do_clear_text_password(MASTER(cp), frontend, 0, protoMajor);
 
-		if (pid >= 0 && REPLICATION)
+		if (pid >= 0 && DUAL_MODE)
 		{
 			pid = do_clear_text_password(SECONDARY(cp), frontend, 0, protoMajor);
 		}
@@ -161,7 +169,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 
 		pid = do_crypt(MASTER(cp), frontend, 0, protoMajor);
 
-		if (pid >= 0 && REPLICATION)
+		if (pid >= 0 && DUAL_MODE)
 		{
 			pid = do_crypt(SECONDARY(cp), frontend, 0, protoMajor);
 		}
@@ -174,7 +182,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 
 		pid = do_md5(MASTER(cp), frontend, 0, protoMajor);
 
-		if (pid >= 0 && REPLICATION)
+		if (pid >= 0 && DUAL_MODE)
 		{
 			pid = do_md5(SECONDARY(cp), frontend, 0, protoMajor);
 		}
@@ -190,55 +198,91 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	 * authentication ok. now read pid and secret key from the
 	 * backend
 	 */
-	kind = pool_read_kind(cp);
-	if (kind < 0)
+	for (;;)
 	{
-		return -1;
-	}
+		kind = pool_read_kind(cp);
+		if (kind < 0)
+		{
+			pool_error("pool_do_auth: failed to read kind before backendkeydata");
+			return -1;
+		}
+		else if (kind == 'K')
+			break;
 
-	/* error response? */
-	if (kind == 'E')
-	{
-		if (protoMajor == PROTO_MAJOR_V2)
-			ErrorResponse(frontend, cp);
-		else
-			SimpleForwardToFrontend(kind, frontend, cp);
-		return -1;
-	}
-	else if (kind != 'K')
-	{
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			/* process parameter status */
-			while (kind == 'S')
+			switch (kind)
 			{
-				if (ParameterStatus(frontend, cp) != POOL_CONTINUE)
+				case 'S':
+					/* process parameter status */
+					if (ParameterStatus(frontend, cp) != POOL_CONTINUE)
 						return -1;
+					pool_flush(frontend);
+					break;
 
-				pool_flush(frontend);
+				case 'N':
+					/* process notice message */
+					if (SimpleForwardToFrontend(kind, frontend, cp))
+						return -1;
+					pool_flush(frontend);
+					break;
 
-				kind = pool_read_kind(cp);
-				if (kind < 0)
-				{
-					pool_error("pool_do_auth: failed to read kind while processing ParamterStatus");
+					/* process error message */
+				case 'E':
+					SimpleForwardToFrontend(kind, frontend, cp);
+					pool_flush(frontend);
 					return -1;
-				}
+					break;
+
+				default:
+					pool_error("pool_do_auth: unknown response \"%c\" before processing BackendKeyData",
+							   kind);
+					return -1;
+					break;
 			}
 		}
 		else
 		{
-			pool_error("pool_do_auth: expect \"K\" got %c", kind);
-			return -1;
+			/* V2 case */
+			switch (kind)
+			{
+				case 'N':
+					/* process notice message */
+					if (NoticeResponse(frontend, cp) != POOL_CONTINUE)
+						return -1;
+					break;
+
+					/* process error message */
+				case 'E':
+					ErrorResponse(frontend, cp);
+					return -1;
+					break;
+
+				default:
+					pool_error("pool_do_auth: unknown response \"%c\" before processing V2 BackendKeyData",
+							   kind);
+					return -1;
+					break;
+			}
 		}
 	}
 
 	/*
 	 * message length (V3 only)
 	 */
-	if (protoMajor == PROTO_MAJOR_V3 && (length = pool_read_message_length(cp)) != 12)
+	if (protoMajor == PROTO_MAJOR_V3)
 	{
-		pool_error("pool_do_auth: invalid messages length(%d) for BackendKeyData", length);
-		return -1;
+		if (kind != 'K')
+		{
+			pool_error("pool_do_auth: expect \"K\" got %c", kind);
+			return -1;
+		}
+
+		if ((length = pool_read_message_length(cp)) != 12)
+		{
+			pool_error("pool_do_auth: invalid messages length(%d) for BackendKeyData", length);
+			return -1;
+		}
 	}
 
 	/*
@@ -253,7 +297,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	pool_read(MASTER(cp), &key, sizeof(key));
 	MASTER_CONNECTION(cp)->key = key;
 
-	if (REPLICATION)
+	if (DUAL_MODE)
 	{
 		pool_read(SECONDARY(cp), &pid1, sizeof(pid1));
 		SECONDARY_CONNECTION(cp)->pid = pid;
@@ -306,18 +350,20 @@ int pool_do_reauth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 
 	if (status == 0)
 	{
+		int msglen;
+
+		pool_write(frontend, "R", 1);
+
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			int msglen;
-
-			pool_write(frontend, "R", 1);
 			msglen = htonl(8);
 			pool_write(frontend, &msglen, sizeof(msglen));
-			msglen = htonl(0);
-			if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
-			{
-				return -1;
-			}
+		}
+
+		msglen = htonl(0);
+		if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
+		{
+			return -1;
 		}
 	}
 	else
@@ -337,6 +383,7 @@ static POOL_STATUS pool_send_auth_ok(POOL_CONNECTION *frontend, int pid, int key
 	char kind;
 	int len;
 
+#ifdef NOT_USED
 	if (protoMajor == PROTO_MAJOR_V2)
 	{
 		/* return "Authentication OK" to the frontend */
@@ -348,6 +395,7 @@ static POOL_STATUS pool_send_auth_ok(POOL_CONNECTION *frontend, int pid, int key
 			return -1;
 		}
 	}
+#endif
 
 	/* send backend key data */
 	kind = 'K';
@@ -492,13 +540,18 @@ static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *fro
 	/* if authenticated, save info */
 	if (!reauth && kind == 0)
 	{
-		if (!backend->issecondary_backend && protoMajor == PROTO_MAJOR_V3)
+		if (!backend->issecondary_backend)
 		{
 			int msglen;
 
 			pool_write(frontend, "R", 1);
-			msglen = htonl(8);
-			pool_write(frontend, &msglen, sizeof(msglen));
+
+			if (protoMajor == PROTO_MAJOR_V3)
+			{
+				msglen = htonl(8);
+				pool_write(frontend, &msglen, sizeof(msglen));
+			}
+
 			msglen = htonl(0);
 			if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
 			{
@@ -656,19 +709,22 @@ static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int rea
 	/* if authenticated, save info */
 	if (!reauth && kind == 0)
 	{
+		int msglen;
+
+		pool_write(frontend, "R", 1);
+
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			int msglen;
-
-			pool_write(frontend, "R", 1);
 			msglen = htonl(8);
 			pool_write(frontend, &msglen, sizeof(msglen));
-			msglen = htonl(0);
-			if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
-			{
-				return -1;
-			}
 		}
+
+		msglen = htonl(0);
+		if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
+		{
+			return -1;
+		}
+
 		backend->auth_kind = 4;
 		backend->pwd_size = ntohl(size) - 4;
 		memcpy(backend->password, password, backend->pwd_size);
@@ -821,19 +877,22 @@ static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reaut
 	/* if authenticated, save info */
 	if (!reauth && kind == 0)
 	{
+		int msglen;
+
+		pool_write(frontend, "R", 1);
+
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			int msglen;
-
-			pool_write(frontend, "R", 1);
 			msglen = htonl(8);
 			pool_write(frontend, &msglen, sizeof(msglen));
-			msglen = htonl(0);
-			if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
-			{
-				return -1;
-			}
 		}
+
+		msglen = htonl(0);
+		if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
+		{
+			return -1;
+		}
+
 		backend->auth_kind = 5;
 		backend->pwd_size = ntohl(size) - 4;
 		memcpy(backend->password, password, backend->pwd_size);
@@ -853,24 +912,26 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 	status = pool_read(MASTER(cp), &length, sizeof(length));
 	if (status < 0)
 	{
-		pool_error("read_message_length: error while reading message length");
+		pool_error("pool_read_message_length: error while reading message length");
 		return -1;
 	}
 	length = ntohl(length);
 
-	if (REPLICATION)
+	pool_debug("pool_read_message_length: lenghth: %d", length);
+
+	if (DUAL_MODE)
 	{
 		status = pool_read(SECONDARY(cp), &length1, sizeof(length1));
 		if (status < 0)
 		{
-			pool_error("read_message_length: error while reading message length from secondary backend");
+			pool_error("pool_read_message_length: error while reading message length from secondary backend");
 			return -1;
 		}
 		length1 = ntohl(length1);
 
 		if (length != length1)
 		{
-			pool_error("read_message_length: length does not match between backends master(%d) secondary(%d)",
+			pool_error("pool_read_message_length: length does not match between backends master(%d) secondary(%d)",
 					   length, length1);
 			return -1;
 		}
@@ -878,13 +939,66 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 
 	if (length < 0)
 	{
-		pool_error("read_message_length: invalid message length (%d)", length);
+		pool_error("pool_read_message_length: invalid message length (%d)", length);
 		return -1;
 	}
 
 	return length;
 }
 
+/*
+ * read message length2 (V3 only)
+ * unlike pool_read_message_length, this returns an array of message length.
+ * the array is in the static storage, thus it will be destroyed by subsequent calls.
+ */
+int *pool_read_message_length2(POOL_CONNECTION_POOL *cp)
+{
+	int status;
+	int length, length1;
+	static int length_array[MAX_CONNECTION_SLOTS];
+
+	status = pool_read(MASTER(cp), &length, sizeof(length));
+	if (status < 0)
+	{
+		pool_error("pool_read_message_length2: error while reading message length");
+		return NULL;
+	}
+	length = ntohl(length);
+
+	pool_debug("pool_read_message_length2: master lenghth: %d", length);
+
+	if (DUAL_MODE)
+	{
+		status = pool_read(SECONDARY(cp), &length1, sizeof(length1));
+		if (status < 0)
+		{
+			pool_error("pool_read_message_length2: error while reading message length from secondary backend");
+			return NULL;
+		}
+		length1 = ntohl(length1);
+
+		if (length != length1)
+		{
+			pool_debug("pool_read_message_length2: length does not match between backends master(%d) secondary(%d)",
+					   length, length1);
+		}
+	}
+	else
+	{
+		length1 = 0;
+	}
+
+	if (length < 0 || length1 < 0)
+	{
+		pool_error("pool_read_message_length2: invalid message length (%d) length1 (%d)", length, length1);
+		return NULL;
+	}
+
+	length_array[0] = length;
+	length_array[1] = length1;
+	return &length_array[0];
+}
+
 signed char pool_read_kind(POOL_CONNECTION_POOL *cp)
 {
 	int status;
@@ -893,22 +1007,22 @@ signed char pool_read_kind(POOL_CONNECTION_POOL *cp)
 	status = pool_read(MASTER(cp), &kind, sizeof(kind));
 	if (status < 0)
 	{
-		pool_error("read_message_kind: error while reading message kind");
+		pool_error("read_kind: error while reading message kind");
 		return -1;
 	}
 
-	if (REPLICATION)
+	if (DUAL_MODE)
 	{
 		status = pool_read(SECONDARY(cp), &kind1, sizeof(kind1));
 		if (status < 0)
 		{
-			pool_error("read_message_kind: error while reading message kind from secondary backend");
+			pool_error("read_kind: error while reading message kind from secondary backend");
 			return -1;
 		}
 
 		if (kind != kind1)
 		{
-			pool_error("read_message_kind: kind does not match between backends master(%d) secondary(%d)",
+			pool_error("read_kind: kind does not match between backends master(%d) secondary(%d)",
 					   kind, kind1);
 			return -1;
 		}
