@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.6 2005/07/18 01:42:48 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.7 2005/10/22 12:57:57 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -63,9 +63,6 @@ static POOL_STATUS CopyDataRows(POOL_CONNECTION *frontend,
 static POOL_STATUS CursorResponse(POOL_CONNECTION *frontend, 
 								  POOL_CONNECTION_POOL *backend);
 
-static POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend, 
-								  POOL_CONNECTION_POOL *backend);
-
 static POOL_STATUS EmptyQueryResponse(POOL_CONNECTION *frontend,
 									  POOL_CONNECTION_POOL *backend);
 
@@ -1116,7 +1113,7 @@ POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend,
 	return POOL_CONTINUE;
 }
 
-static POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend, 
 								  POOL_CONNECTION_POOL *backend)
 {
 	char *string, *string1;
